@@ -1,7 +1,7 @@
 // qjackctlConnect.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,6 +25,36 @@
 #include <qmessagebox.h>
 #include <qtimer.h>
 
+//----------------------------------------------------------------------
+// class qjackctlConnectToolTip -- custom list view tooltips.
+
+// Constructor.
+qjackctlConnectToolTip::qjackctlConnectToolTip ( qjackctlClientListView *pListView )
+	: QToolTip(pListView->viewport())
+{
+	m_pListView = pListView;
+}
+
+// Tooltip handler.
+void qjackctlConnectToolTip::maybeTip ( const QPoint& pos )
+{
+	QListViewItem *pItem = m_pListView->itemAt(pos);
+	if (pItem == 0)
+		return;
+
+	QRect rect(m_pListView->itemRect(pItem));
+	if (!rect.isValid())
+		return;
+
+	if (pItem->rtti() == QJACKCTL_CLIENTITEM) {
+		qjackctlClientItem *pClient = (qjackctlClientItem *) pItem;
+		QToolTip::tip(rect, pClient->clientName());
+	} else {
+		qjackctlPortItem *pPort = (qjackctlPortItem *) pItem;
+		QToolTip::tip(rect, pPort->portName());
+	}
+}
+
 
 //----------------------------------------------------------------------
 // class qjackctlPortItem -- Port list item.
@@ -45,12 +75,21 @@ qjackctlPortItem::qjackctlPortItem ( qjackctlClientItem *pClient, const QString&
     QListViewItem::setDropEnabled(true);
 
     m_connects.setAutoDelete(false);
+
+	// Check aliasing...
+	qjackctlConnectAlias *pAliases = ((pClient->clientList())->listView())->aliases();
+	if (pAliases) {
+		QListViewItem::setText(0, pAliases->portAlias(pClient->clientName(), sPortName));
+		QListViewItem::setRenameEnabled(0, ((pClient->clientList())->listView())->renameEnabled());
+	}
 }
 
 // Default destructor.
 qjackctlPortItem::~qjackctlPortItem (void)
 {
     m_pClient->ports().remove(this);
+	for (qjackctlPortItem *pPort = m_connects.first(); pPort; pPort = m_connects.next())
+		pPort->removeConnect(this);
     m_connects.clear();
 }
 
@@ -178,6 +217,13 @@ void qjackctlPortItem::paintCell( QPainter *p, const QColorGroup& cg, int column
 }
 
 
+// Special port name sorting virtual comparator.
+int qjackctlPortItem::compare (QListViewItem* pPortItem, int iColumn, bool bAscending) const
+{
+	return qjackctlClientListView::compare(text(iColumn), pPortItem->text(iColumn), bAscending);
+}
+
+
 //----------------------------------------------------------------------
 // class qjackctlClientItem -- Jack client list item.
 //
@@ -199,6 +245,13 @@ qjackctlClientItem::qjackctlClientItem ( qjackctlClientList *pClientList, const
     QListViewItem::setDropEnabled(true);
 
 //  QListViewItem::setSelectable(false);
+
+	// Check aliasing...
+	qjackctlConnectAlias *pAliases = (pClientList->listView())->aliases();
+	if (pAliases) {
+	    QListViewItem::setText(0, pAliases->clientAlias(sClientName));
+        QListViewItem::setRenameEnabled(0, (pClientList->listView())->renameEnabled());
+	}
 }
 
 // Default destructor.
@@ -222,7 +275,7 @@ qjackctlPortItem *qjackctlClientItem::findPort (const QString& sPortName)
 
 
 // Client list accessor.
-qjackctlClientList *qjackctlClientItem::clientlist (void)
+qjackctlClientList *qjackctlClientItem::clientList (void)
 {
     return m_pClientList;
 }
@@ -315,6 +368,13 @@ void qjackctlClientItem::paintCell( QPainter *p, const QColorGroup& cg, int colu
 }
 
 
+// Special client name sorting virtual comparator.
+int qjackctlClientItem::compare (QListViewItem* pClientItem, int iColumn, bool bAscending) const
+{
+	return qjackctlClientListView::compare(text(iColumn), pClientItem->text(iColumn), bAscending);
+}
+
+
 //----------------------------------------------------------------------
 // qjackctlClientList -- Client list.
 //
@@ -463,6 +523,11 @@ qjackctlClientListView::qjackctlClientListView ( qjackctlConnectView *pConnectVi
     m_iAutoOpenTimeout = 0;
     m_pDragDropItem    = 0;
     
+	m_pAliases = 0;
+	m_bRenameEnabled = false;
+
+	m_pToolTip = new qjackctlConnectToolTip(this);
+
     if (bReadable)
         QListView::addColumn(tr("Readable Clients") + " / " + tr("Output Ports"));
     else
@@ -477,14 +542,21 @@ qjackctlClientListView::qjackctlClientListView ( qjackctlConnectView *pConnectVi
     QListView::setAcceptDrops(true);
     QListView::setDragAutoScroll(true);
     QListView::setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
+
+	QListView::setShowToolTips(false);
     
     setAutoOpenTimeout(800);
+    
+    QObject::connect(this, SIGNAL(itemRenamed(QListViewItem*,int)),
+		this, SLOT(renamedSlot(QListViewItem*,int)));
 }
 
 // Default destructor.
 qjackctlClientListView::~qjackctlClientListView (void)
 {
     setAutoOpenTimeout(0);
+
+	delete m_pToolTip;
 }
 
 
@@ -511,6 +583,79 @@ int qjackctlClientListView::autoOpenTimeout (void)
 }
 
 
+// Aliasing support methods.
+void qjackctlClientListView::setAliases ( qjackctlConnectAlias *pAliases, bool bRenameEnabled )
+{
+	m_pAliases = pAliases;
+	m_bRenameEnabled = bRenameEnabled;
+
+	// Update all current listview items, if any.
+	QListViewItemIterator iter(this);
+	while (iter.current()) {
+		QListViewItem *pItem = iter.current();
+		if (pItem->rtti() == QJACKCTL_CLIENTITEM) {
+			qjackctlClientItem *pClient = (qjackctlClientItem *) pItem;
+			if (m_pAliases) {
+				pClient->setText(0, m_pAliases->clientAlias(pClient->clientName()));
+				pClient->setRenameEnabled(0, m_bRenameEnabled);
+			} else {
+				pClient->setText(0, pClient->clientName());
+				pClient->setRenameEnabled(0, false);
+			}
+		} else {
+			qjackctlPortItem *pPort = (qjackctlPortItem *) pItem;
+			if (m_pAliases) {
+				pPort->setText(0, m_pAliases->portAlias(pPort->clientName(), pPort->portName()));
+				pPort->setRenameEnabled(0, m_bRenameEnabled);
+			} else {
+				pPort->setText(0, pPort->portName());
+				pPort->setRenameEnabled(0, false);
+			}
+		}
+		++iter;
+	}
+}
+
+qjackctlConnectAlias *qjackctlClientListView::aliases (void)
+{
+	return m_pAliases;
+}
+
+bool qjackctlClientListView::renameEnabled (void)
+{
+	return m_bRenameEnabled;
+}
+
+// In-place aliasing slot.
+void qjackctlClientListView::startRenameSlot (void)
+{
+	QListViewItem *pItem = selectedItem();
+	if (pItem)
+	    pItem->startRename(0);
+}
+
+
+// In-place aliasing slot.
+void qjackctlClientListView::renamedSlot ( QListViewItem *pItem, int )
+{
+	if (pItem && m_pAliases) {
+		const QString& sText = pItem->text(0);
+	    if (pItem->rtti() == QJACKCTL_CLIENTITEM) {
+			qjackctlClientItem *pClient = (qjackctlClientItem *) pItem;
+		    m_pAliases->setClientAlias(pClient->clientName(), sText);
+			if (sText.isEmpty())
+				pClient->setText(0, pClient->clientName());
+		} else {
+			qjackctlPortItem *pPort = (qjackctlPortItem *) pItem;
+		    m_pAliases->setPortAlias(pPort->clientName(), pPort->portName(), sText);
+			if (sText.isEmpty())
+				pPort->setText(0, pPort->portName());
+		}
+		m_pConnectView->setDirty(true);
+	}
+}
+
+
 // Auto-open timer slot.
 void qjackctlClientListView::timeoutSlot (void)
 {
@@ -617,7 +762,88 @@ QDragObject *qjackctlClientListView::dragObject (void)
 // Context menu request event handler.
 void qjackctlClientListView::contextMenuEvent ( QContextMenuEvent *pContextMenuEvent )
 {
-    m_pConnectView->contextMenu(pContextMenuEvent->globalPos());
+    qjackctlConnect *pConnect = m_pConnectView->binding();
+    if (pConnect == 0)
+        return;
+
+    int iItemID;
+    QPopupMenu* pContextMenu = new QPopupMenu(this);
+
+    iItemID = pContextMenu->insertItem(tr("&Connect"), pConnect, SLOT(connectSelected()), tr("Alt+C", "Connect"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canConnectSelected());
+    iItemID = pContextMenu->insertItem(tr("&Disconnect"), pConnect, SLOT(disconnectSelected()), tr("Alt+D", "Disconnect"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectSelected());
+    iItemID = pContextMenu->insertItem(tr("Disconnect &All"), pConnect, SLOT(disconnectAll()), tr("Alt+A", "Disconect All"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectAll());
+	if (m_bRenameEnabled) {
+		pContextMenu->insertSeparator();
+		iItemID = pContextMenu->insertItem(tr("Re&name"), this, SLOT(startRenameSlot()), tr("Alt+N", "Rename"));
+		pContextMenu->setItemEnabled(iItemID, selectedItem() && selectedItem()->renameEnabled(0));
+	}
+    pContextMenu->insertSeparator();
+    iItemID = pContextMenu->insertItem(tr("&Refresh"), pConnect, SLOT(refresh()), tr("Alt+R", "Refresh"));
+
+    pContextMenu->exec(pContextMenuEvent->globalPos());
+
+    delete pContextMenu;
+}
+
+
+// Natural decimal sorting comparator.
+int qjackctlClientListView::compare (const QString& s1, const QString& s2, bool bAscending)
+{
+    int ich1, ich2;
+
+    int cch1 = s1.length();
+    int cch2 = s2.length();
+
+    for (ich1 = ich2 = 0; ich1 < cch1 && ich2 < cch2; ich1++, ich2++) {
+
+        // Skip (white)spaces...
+        while (s1.at(ich1).isSpace())
+            ich1++;
+        while (s2.at(ich2).isSpace())
+            ich2++;
+
+		// Normalize (to uppercase) the next characters...
+        QChar ch1 = s1.at(ich1).upper();
+        QChar ch2 = s2.at(ich2).upper();
+
+        if (ch1.isDigit() && ch2.isDigit()) {
+            // Find the whole length numbers...
+            int iDigits1 = ich1++;
+            while (s1.at(ich1).isDigit())
+                ich1++;
+            int iDigits2 = ich2++;
+            while (s2.at(ich2).isDigit())
+                ich2++;
+            // Compare as natural decimal-numbers...
+            int iNumber1 = s1.mid(iDigits1, ich1 - iDigits1).toInt();
+            int iNumber2 = s2.mid(iDigits2, ich2 - iDigits2).toInt();
+            if (iNumber1 < iNumber2)
+                return (bAscending ? -1 :  1);
+            else if (iNumber1 > iNumber2)
+                return (bAscending ?  1 : -1);
+            // Go on with this next char...
+            ch1 = s1.at(ich1).upper();
+            ch2 = s2.at(ich2).upper();
+        }
+
+        // Compare this char...
+        if (ch1 < ch2)
+            return (bAscending ? -1 :  1);
+        else if (ch1 > ch2)
+            return (bAscending ?  1 : -1);
+    }
+
+    // Both strings seem to match, but longer is greater.
+    if (cch1 < cch2)
+        return (bAscending ? -1 :  1);
+    else if (cch1 > cch2)
+        return (bAscending ?  1 : -1);
+
+    // Exact match.
+    return 0;
 }
 
 
@@ -755,7 +981,26 @@ void qjackctlConnectorView::resizeEvent ( QResizeEvent * )
 // Context menu request event handler.
 void qjackctlConnectorView::contextMenuEvent ( QContextMenuEvent *pContextMenuEvent )
 {
-    m_pConnectView->contextMenu(pContextMenuEvent->globalPos());
+    qjackctlConnect *pConnect = m_pConnectView->binding();
+    if (pConnect == 0)
+        return;
+
+    int iItemID;
+    QPopupMenu* pContextMenu = new QPopupMenu(this);
+
+    iItemID = pContextMenu->insertItem(tr("&Connect"), pConnect, SLOT(connectSelected()), tr("Alt+C", "Connect"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canConnectSelected());
+    iItemID = pContextMenu->insertItem(tr("&Disconnect"), pConnect, SLOT(disconnectSelected()), tr("Alt+D", "Disconnect"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectSelected());
+    iItemID = pContextMenu->insertItem(tr("Disconnect &All"), pConnect, SLOT(disconnectAll()), tr("Alt+A", "Disconect All"));
+    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectAll());
+
+    pContextMenu->insertSeparator();
+    iItemID = pContextMenu->insertItem(tr("&Refresh"), pConnect, SLOT(refresh()), tr("Alt+R", "Refresh"));
+
+    pContextMenu->exec(pContextMenuEvent->globalPos());
+
+    delete pContextMenu;
 }
 
 
@@ -797,7 +1042,9 @@ qjackctlConnectView::qjackctlConnectView ( QWidget *pParent, const char *pszName
     QObject::connect(m_pIListView, SIGNAL(collapsed(QListViewItem *)), m_pConnectorView, SLOT(listViewChanged(QListViewItem *)));
     QObject::connect(m_pIListView, SIGNAL(contentsMoving(int, int)),   m_pConnectorView, SLOT(contentsMoved(int, int)));
 
-#if QT_VERSION >= 0x030200    
+    m_bDirty = false;
+
+#if QT_VERSION >= 0x030200
     QSplitter::setChildrenCollapsible(false);
 #endif	
 }
@@ -809,32 +1056,6 @@ qjackctlConnectView::~qjackctlConnectView (void)
 }
 
 
-// Common context menu slot.
-void qjackctlConnectView::contextMenu ( const QPoint& pos )
-{
-    qjackctlConnect *pConnect = binding();
-    if (pConnect == 0)
-        return;
-
-    int iItemID;
-    QPopupMenu* pContextMenu = new QPopupMenu(this);
-
-    iItemID = pContextMenu->insertItem(tr("&Connect"), pConnect, SLOT(connectSelected()), tr("Alt+C", "Connect"));
-    pContextMenu->setItemEnabled(iItemID, pConnect->canConnectSelected());
-    iItemID = pContextMenu->insertItem(tr("&Disconnect"), pConnect, SLOT(disconnectSelected()), tr("Alt+D", "Disconnect"));
-    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectSelected());
-    iItemID = pContextMenu->insertItem(tr("Disconnect &All"), pConnect, SLOT(disconnectAll()), tr("Alt+A", "Disconect All"));
-    pContextMenu->setItemEnabled(iItemID, pConnect->canDisconnectAll());
-
-    pContextMenu->insertSeparator();
-    iItemID = pContextMenu->insertItem(tr("&Refresh"), pConnect, SLOT(refresh()), tr("Alt+R", "Refresh"));
-
-    pContextMenu->exec(pos);
-    
-    delete pContextMenu;
-}
-
-
 // Connect binding methods.
 void qjackctlConnectView::setBinding ( qjackctlConnect *pConnect )
 {
@@ -899,6 +1120,20 @@ int qjackctlConnectView::iconSize (void)
 }
 
 
+// Dirty flag methods.
+void qjackctlConnectView::setDirty ( bool bDirty )
+{
+    m_bDirty = bDirty;
+    if (bDirty)
+        emit contentsChanged();
+}
+
+bool qjackctlConnectView::dirty (void)
+{
+    return m_bDirty;
+}
+
+
 //----------------------------------------------------------------------
 // qjackctlConnect -- Output-to-Input client/ports connection object.
 //
@@ -1111,7 +1346,7 @@ bool qjackctlConnect::connectSelectedEx (void)
             connectPortsEx(pOPort, pIPort);
         }
     }
-    
+
     return true;
 }
 
@@ -1248,7 +1483,7 @@ bool qjackctlConnect::disconnectSelectedEx (void)
             disconnectPortsEx(pOPort, pIPort);
         }
     }
-    
+
     return true;
 }
 
@@ -1331,16 +1566,16 @@ void qjackctlConnect::updateContents ( bool bClear )
     int iDirtyCount = 0;
 
     if (startMutex()) {
-        // Do we do a complete rebuild?
-        if (bClear) {
-            (m_pOClientList->listView())->clear();
-            (m_pIClientList->listView())->clear();
-            updateIconPixmaps();
-        }
-        // Add (newer) client:ports and respective connections...
-        iDirtyCount += m_pOClientList->updateClientPorts();
-        iDirtyCount += m_pIClientList->updateClientPorts();
-        updateConnections();
+    // Do we do a complete rebuild?
+    if (bClear) {
+        (m_pOClientList->listView())->clear();
+        (m_pIClientList->listView())->clear();
+        updateIconPixmaps();
+    }
+    // Add (newer) client:ports and respective connections...
+    iDirtyCount += m_pOClientList->updateClientPorts();
+    iDirtyCount += m_pIClientList->updateClientPorts();
+    updateConnections();
         endMutex();
     }
 
