@@ -1,7 +1,7 @@
 // qjackctlPatchbay.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <qpixmap.h>
 #include <qpainter.h>
 #include <qpopupmenu.h>
+#include <qtooltip.h>
 
 // Our external patchbay models.
 #include "qjackctlPatchbayRack.h"
@@ -57,6 +58,26 @@ class qjackctlPatchbay;
 #define QJACKCTL_XPM_PIXMAPS        6
 
 
+// Custom tooltip class.
+class qjackctlPatchbayToolTip : public QToolTip
+{
+public:
+
+	// Constructor.
+	qjackctlPatchbayToolTip(qjackctlSocketListView *pListView);
+
+protected:
+
+	// Tooltip handler.
+	void maybeTip(const QPoint& pos);
+
+private:
+
+	// The actual parent widget holder.
+	qjackctlSocketListView *m_pListView;
+};
+
+
 // Patchbay plug (port) list item.
 class qjackctlPlugItem : public QListViewItem
 {
@@ -209,9 +230,6 @@ public:
     // Retrieve a context pixmap.
     QPixmap& pixmap(int iPixmap);
 
-    // Escape and format a string as a regular expresion
-    static QString escapeRegExpDigits(const QString& s, unsigned int iThreshold = 2);
-
 public slots:
 
     // Socket item interactivity methods.
@@ -292,6 +310,10 @@ private:
     QTimer *m_pAutoOpenTimer;
     // Item we'll eventually drop something.
     QListViewItem *m_pDragDropItem;
+
+
+	// Listview item tooltip.
+	qjackctlPatchbayToolTip *m_pToolTip;
 };
 
 
