@@ -1,7 +1,7 @@
 // qjackctlSetup.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -77,6 +77,9 @@ qjackctlSetup::qjackctlSetup (void)
     bServerConfig            = m_settings.readBoolEntry("/ServerConfig", true);
     sServerConfigName        = m_settings.readEntry("/ServerConfigName", ".jackdrc");
     bServerConfigTemp        = m_settings.readBoolEntry("/ServerConfigTemp", false);
+    bQueryShutdown           = m_settings.readBoolEntry("/QueryShutdown", true);
+    bAliasesEnabled          = m_settings.readBoolEntry("/AliasesEnabled", false);
+    bAliasesEditing          = m_settings.readBoolEntry("/AliasesEditing", false);
     m_settings.endGroup();
 
     m_settings.beginGroup("/Defaults");
@@ -139,6 +142,9 @@ qjackctlSetup::~qjackctlSetup (void)
     m_settings.writeEntry("/ServerConfig",            bServerConfig);
     m_settings.writeEntry("/ServerConfigName",        sServerConfigName);
     m_settings.writeEntry("/ServerConfigTemp",        bServerConfigTemp);
+    m_settings.writeEntry("/QueryShutdown",           bQueryShutdown);
+    m_settings.writeEntry("/AliasesEnabled",          bAliasesEnabled);
+    m_settings.writeEntry("/AliasesEditing",          bAliasesEditing);
     m_settings.endGroup();
 
     m_settings.beginGroup("/Defaults");
@@ -149,6 +155,63 @@ qjackctlSetup::~qjackctlSetup (void)
 }
 
 
+//---------------------------------------------------------------------------
+// Aliases preset management methods.
+
+bool qjackctlSetup::loadAliases ( const QString& sPreset )
+{
+    QString sSuffix;
+    if (sPreset != sDefPresetName && !sPreset.isEmpty()) {
+        sSuffix = "/" + sPreset;
+        // Check if on list.
+        if (presets.find(sPreset) == presets.end())
+            return false;
+    }
+
+	// Load preset aliases...
+	const QString sAliasesKey = "/Aliases" + sSuffix;
+    m_settings.beginGroup(sAliasesKey);
+	  m_settings.beginGroup("/Jack");
+	 	aliasJackOutputs.loadSettings(m_settings, "/Outputs");
+	 	aliasJackInputs.loadSettings(m_settings, "/Inputs");
+	  m_settings.endGroup();
+	  m_settings.beginGroup("/Alsa");
+	 	aliasAlsaOutputs.loadSettings(m_settings, "/Outputs");
+	 	aliasAlsaInputs.loadSettings(m_settings, "/Inputs");
+	  m_settings.endGroup();
+    m_settings.endGroup();
+
+    return true;
+}
+
+bool qjackctlSetup::saveAliases ( const QString& sPreset )
+{
+    QString sSuffix;
+    if (sPreset != sDefPresetName && !sPreset.isEmpty()) {
+        sSuffix = "/" + sPreset;
+        // Append to list if not already.
+        if (presets.find(sPreset) == presets.end())
+            presets.prepend(sPreset);
+    }
+
+	// Save preset aliases...
+	const QString sAliasesKey = "/Aliases" + sSuffix;
+	deleteKey(sAliasesKey);
+    m_settings.beginGroup(sAliasesKey);
+	  m_settings.beginGroup("/Jack");
+	 	aliasJackOutputs.saveSettings(m_settings, "/Outputs");
+	 	aliasJackInputs.saveSettings(m_settings, "/Inputs");
+	  m_settings.endGroup();
+	  m_settings.beginGroup("/Alsa");
+	 	aliasAlsaOutputs.saveSettings(m_settings, "/Outputs");
+	 	aliasAlsaInputs.saveSettings(m_settings, "/Inputs");
+	  m_settings.endGroup();
+    m_settings.endGroup();
+
+    return true;
+}
+
+
 //---------------------------------------------------------------------------
 // Preset management methods.
 
@@ -163,12 +226,13 @@ bool qjackctlSetup::loadPreset ( qjackctlPreset& preset, const QString& sPreset
     }
 
     m_settings.beginGroup("/Settings" + sSuffix);
-    preset.sServer      = m_settings.readEntry("/Server", "jackstart");
+    preset.sServer      = m_settings.readEntry("/Server", "jackd");
     preset.bRealtime    = m_settings.readBoolEntry("/Realtime", true);
     preset.bSoftMode    = m_settings.readBoolEntry("/SoftMode", false);
     preset.bMonitor     = m_settings.readBoolEntry("/Monitor", false);
     preset.bShorts      = m_settings.readBoolEntry("/Shorts", false);
     preset.bNoMemLock   = m_settings.readBoolEntry("/NoMemLock", false);
+    preset.bUnlockMem   = m_settings.readBoolEntry("/UnlockMem", false);
     preset.bHWMon       = m_settings.readBoolEntry("/HWMon", false);
     preset.bHWMeter     = m_settings.readBoolEntry("/HWMeter", false);
     preset.bIgnoreHW    = m_settings.readBoolEntry("/IgnoreHW", false);
@@ -213,6 +277,7 @@ bool qjackctlSetup::savePreset ( qjackctlPreset& preset, const QString& sPreset
     m_settings.writeEntry("/Monitor",     preset.bMonitor);
     m_settings.writeEntry("/Shorts",      preset.bShorts);
     m_settings.writeEntry("/NoMemLock",   preset.bNoMemLock);
+    m_settings.writeEntry("/UnlockMem",   preset.bUnlockMem);
     m_settings.writeEntry("/HWMon",       preset.bHWMon);
     m_settings.writeEntry("/HWMeter",     preset.bHWMeter);
     m_settings.writeEntry("/IgnoreHW",    preset.bIgnoreHW);
@@ -250,6 +315,7 @@ bool qjackctlSetup::deletePreset ( const QString& sPreset )
             return false;
         presets.remove(iter);
         deleteKey("/Settings" + sSuffix);
+    	deleteKey("/Aliases" + sSuffix);
     }
     return true;
 }
