@@ -1,7 +1,7 @@
 // qjackctlSetup.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,10 +22,10 @@
 #ifndef __qjackctlSetup_h
 #define __qjackctlSetup_h
 
-#include <qstring.h>
-#include <qsettings.h>
 #include <qcombobox.h>
 
+#include "qjackctlConnectAlias.h"
+
 // Audio mode combobox item indexes.
 #define QJACKCTL_DUPLEX     0
 #define QJACKCTL_CAPTURE    1
@@ -46,6 +46,7 @@ struct qjackctlPreset
     bool    bMonitor;
     bool    bShorts;
     bool    bNoMemLock;
+    bool    bUnlockMem;
     bool    bHWMon;
     bool    bHWMeter;
     bool    bIgnoreHW;
@@ -132,10 +133,23 @@ public:
     bool    bServerConfig;
     QString sServerConfigName;
     bool    bServerConfigTemp;
+    bool    bQueryShutdown;
+	bool    bAliasesEnabled;
+	bool    bAliasesEditing;
 
     // Defaults...
     QString sPatchbayPath;
 
+	// Aliases containers.
+	qjackctlConnectAlias aliasJackOutputs;
+	qjackctlConnectAlias aliasJackInputs;
+	qjackctlConnectAlias aliasAlsaOutputs;
+	qjackctlConnectAlias aliasAlsaInputs;
+
+    // Aliases preset management methods.
+    bool loadAliases(const QString& sPreset);
+    bool saveAliases(const QString& sPreset);
+
     // Preset management methods.
     bool loadPreset(qjackctlPreset& preset, const QString& sPreset);
     bool savePreset(qjackctlPreset& preset, const QString& sPreset);
