@@ -2,7 +2,7 @@
 //
 // ui.h extension file, included from the uic-generated form implementation.
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,6 +29,8 @@
 
 #include "config.h"
 
+#include <alsa/asoundlib.h>
+
 
 // Kind of constructor.
 void qjackctlSetupForm::init (void)
@@ -48,7 +50,6 @@ void qjackctlSetupForm::init (void)
     WordLengthComboBox->setValidator(new QIntValidator(WordLengthComboBox));
     TimeoutComboBox->setValidator(new QIntValidator(TimeoutComboBox));
     TimeRefreshComboBox->setValidator(new QIntValidator(TimeRefreshComboBox));
-    StartDelayComboBox->setValidator(new QIntValidator(StartDelayComboBox));
     PortMaxComboBox->setValidator(new QIntValidator(PortMaxComboBox));
     MessagesLimitLinesComboBox->setValidator(new QIntValidator(MessagesLimitLinesComboBox));
 
@@ -150,6 +151,9 @@ void qjackctlSetupForm::setup ( qjackctlSetup *pSetup )
     ServerConfigCheckBox->setChecked(m_pSetup->bServerConfig);
     ServerConfigNameComboBox->setCurrentText(m_pSetup->sServerConfigName);
     ServerConfigTempCheckBox->setChecked(m_pSetup->bServerConfigTemp);
+    QueryShutdownCheckBox->setChecked(m_pSetup->bQueryShutdown);
+    AliasesEnabledCheckBox->setChecked(m_pSetup->bAliasesEnabled);
+    AliasesEditingCheckBox->setChecked(m_pSetup->bAliasesEditing);
 
 #ifndef CONFIG_SYSTEM_TRAY
     SystemTrayCheckBox->setChecked(false);
@@ -182,6 +186,7 @@ void qjackctlSetupForm::changePreset ( const QString& sPreset )
         MonitorCheckBox->setChecked(preset.bMonitor);
         ShortsCheckBox->setChecked(preset.bShorts);
         NoMemLockCheckBox->setChecked(preset.bNoMemLock);
+        UnlockMemCheckBox->setChecked(preset.bUnlockMem);
         HWMonCheckBox->setChecked(preset.bHWMon);
         HWMeterCheckBox->setChecked(preset.bHWMeter);
         IgnoreHWCheckBox->setChecked(preset.bIgnoreHW);
@@ -193,7 +198,10 @@ void qjackctlSetupForm::changePreset ( const QString& sPreset )
         WaitComboBox->setCurrentText(QString::number(preset.iWait));
         ChanSpinBox->setValue(preset.iChan);
         DriverComboBox->setCurrentText(preset.sDriver);
-        InterfaceComboBox->setCurrentText(preset.sInterface);
+        if (preset.sInterface.isEmpty())
+            InterfaceComboBox->setCurrentText(m_pSetup->sDefPresetName);
+        else
+            InterfaceComboBox->setCurrentText(preset.sInterface);
         AudioComboBox->setCurrentItem(preset.iAudio);
         DitherComboBox->setCurrentItem(preset.iDither);
         TimeoutComboBox->setCurrentText(QString::number(preset.iTimeout));
@@ -207,7 +215,7 @@ void qjackctlSetupForm::changePreset ( const QString& sPreset )
             OutDeviceComboBox->setCurrentText(preset.sOutDevice);
         InChannelsSpinBox->setValue(preset.iInChannels);
         OutChannelsSpinBox->setValue(preset.iOutChannels);
-        StartDelayComboBox->setCurrentText(QString::number(preset.iStartDelay));
+        StartDelaySpinBox->setValue(preset.iStartDelay);
         VerboseCheckBox->setChecked(preset.bVerbose);
         PortMaxComboBox->setCurrentText(QString::number(preset.iPortMax));
         // Reset dirty flag.
@@ -232,6 +240,7 @@ bool qjackctlSetupForm::savePreset ( const QString& sPreset )
     preset.bMonitor     = MonitorCheckBox->isChecked();
     preset.bShorts      = ShortsCheckBox->isChecked();
     preset.bNoMemLock   = NoMemLockCheckBox->isChecked();
+    preset.bUnlockMem   = UnlockMemCheckBox->isChecked();
     preset.bHWMon       = HWMonCheckBox->isChecked();
     preset.bHWMeter     = HWMeterCheckBox->isChecked();
     preset.bIgnoreHW    = IgnoreHWCheckBox->isChecked();
@@ -251,9 +260,11 @@ bool qjackctlSetupForm::savePreset ( const QString& sPreset )
     preset.sOutDevice   = OutDeviceComboBox->currentText();
     preset.iInChannels  = InChannelsSpinBox->value();
     preset.iOutChannels = OutChannelsSpinBox->value();
-    preset.iStartDelay  = StartDelayComboBox->currentText().toInt();
+    preset.iStartDelay  = StartDelaySpinBox->value();
     preset.bVerbose     = VerboseCheckBox->isChecked();
     preset.iPortMax     = PortMaxComboBox->currentText().toInt();
+    if (preset.sInterface == m_pSetup->sDefPresetName || preset.sInterface.isEmpty())
+        preset.sInterface = "hw:0";
     if (preset.sInDevice == m_pSetup->sDefPresetName)
         preset.sInDevice = QString::null;
     if (preset.sOutDevice == m_pSetup->sDefPresetName)
@@ -385,10 +396,10 @@ void qjackctlSetupForm::computeLatency (void)
 void qjackctlSetupForm::changeDriverAudio ( const QString& sDriver, int iAudio )
 {
     bool bOss        = (sDriver == "oss");
-    bool bAlsa       = (sDriver == "alsa" || sDriver == "usx2y");
+    bool bAlsa       = (sDriver == "alsa");
     bool bInEnabled  = false;
     bool bOutEnabled = false;
-    
+
     switch (iAudio) {
       case QJACKCTL_DUPLEX:
         bInEnabled  = (bOss || bAlsa);
@@ -404,8 +415,10 @@ void qjackctlSetupForm::changeDriverAudio ( const QString& sDriver, int iAudio )
 
     InDeviceTextLabel->setEnabled(bInEnabled);
     InDeviceComboBox->setEnabled(bInEnabled);
+    InDevicePushButton->setEnabled(bInEnabled && bAlsa);
     OutDeviceTextLabel->setEnabled(bOutEnabled);
     OutDeviceComboBox->setEnabled(bOutEnabled);
+    OutDevicePushButton->setEnabled(bOutEnabled && bAlsa);
 
     InChannelsTextLabel->setEnabled(bInEnabled || (bAlsa && iAudio != QJACKCTL_PLAYBACK));
     InChannelsSpinBox->setEnabled(bInEnabled || (bAlsa && iAudio != QJACKCTL_PLAYBACK));
@@ -426,7 +439,7 @@ void qjackctlSetupForm::changeDriver ( const QString& sDriver )
 {
     bool bDummy     = (sDriver == "dummy");
     bool bOss       = (sDriver == "oss");
-    bool bAlsa      = (sDriver == "alsa" || sDriver == "usx2y");
+    bool bAlsa      = (sDriver == "alsa");
     bool bPortaudio = (sDriver == "portaudio");
 
     SoftModeCheckBox->setEnabled(bAlsa);
@@ -449,13 +462,22 @@ void qjackctlSetupForm::changeDriver ( const QString& sDriver )
     ChanTextLabel->setEnabled(bPortaudio);
     ChanSpinBox->setEnabled(bPortaudio);
 
-    InterfaceTextLabel->setEnabled(bAlsa);
-    InterfaceComboBox->setEnabled(bAlsa);
+	int  iAudio   = AudioComboBox->currentItem();
+	bool bEnabled = bAlsa;
+    if (bEnabled && iAudio == QJACKCTL_DUPLEX) {
+        const QString& sInDevice  = InDeviceComboBox->currentText();
+        const QString& sOutDevice = OutDeviceComboBox->currentText();
+		bEnabled = (sInDevice.isEmpty()  || sInDevice  == m_pSetup->sDefPresetName ||
+					sOutDevice.isEmpty() || sOutDevice == m_pSetup->sDefPresetName);
+	}
+	InterfaceTextLabel->setEnabled(bEnabled);
+	InterfaceComboBox->setEnabled(bEnabled);
+	InterfacePushButton->setEnabled(bEnabled);
 
     DitherTextLabel->setEnabled(bAlsa || bPortaudio);
     DitherComboBox->setEnabled(bAlsa || bPortaudio);
 
-    changeDriverAudio(sDriver, AudioComboBox->currentItem());
+    changeDriverAudio(sDriver, iAudio);
 }
 
 
@@ -472,6 +494,8 @@ void qjackctlSetupForm::stabilizeForm (void)
         PresetDeletePushButton->setEnabled(false);
     }
 
+    UnlockMemCheckBox->setEnabled(!NoMemLockCheckBox->isChecked());
+
     bool bEnabled = RealtimeCheckBox->isChecked();
     PriorityTextLabel->setEnabled(bEnabled);
     PrioritySpinBox->setEnabled(bEnabled);
@@ -512,12 +536,111 @@ void qjackctlSetupForm::stabilizeForm (void)
     ServerConfigNameComboBox->setEnabled(bEnabled);
     ServerConfigTempCheckBox->setEnabled(bEnabled);
 
+    AliasesEditingCheckBox->setEnabled(AliasesEnabledCheckBox->isChecked());
+
     changeDriver(DriverComboBox->currentText());
 }
 
 
+// Device selection menu executive.
+void qjackctlSetupForm::deviceMenu( QLineEdit *pLineEdit,
+	QPushButton *pPushButton, int iAudio )
+{
+	// FIXME: Only valid for ALSA devices,
+	// for the time being...
+	if (DriverComboBox->currentText() != "alsa")
+	    return;
+	
+    QPopupMenu* pContextMenu = new QPopupMenu(this);
+	QString sName, sText;
+
+	// Enumerate just the ALSA cards and PCM harfware devices...
+	snd_ctl_t *handle;
+	snd_ctl_card_info_t *info;
+	snd_pcm_info_t *pcminfo;
+	
+	snd_ctl_card_info_alloca(&info);
+	snd_pcm_info_alloca(&pcminfo);
+
+	int iCards = 0;
+	int iCard = -1;
+	while (snd_card_next(&iCard) >= 0 && iCard >= 0) {
+		sName = "hw:" + QString::number(iCard);
+		if (snd_ctl_open(&handle, sName.latin1(), 0) >= 0
+			&& snd_ctl_card_info(handle, info) >= 0) {
+			if (iCards > 0)
+    			pContextMenu->insertSeparator();
+			sText  = sName + '\t';
+			sText += snd_ctl_card_info_get_name(info);
+    		pContextMenu->insertItem(sText);
+			int iDevice = -1;
+			while (snd_ctl_pcm_next_device(handle, &iDevice) >= 0
+				&& iDevice >= 0) {
+				snd_pcm_info_set_device(pcminfo, iDevice);
+				snd_pcm_info_set_subdevice(pcminfo, 0);
+				snd_pcm_info_set_stream(pcminfo,
+					iAudio == QJACKCTL_CAPTURE ?
+						SND_PCM_STREAM_CAPTURE :
+						SND_PCM_STREAM_PLAYBACK);
+				if (snd_ctl_pcm_info(handle, pcminfo) >= 0) {
+					sText  = sName + ',' + QString::number(iDevice) + '\t';
+					sText += snd_pcm_info_get_name(pcminfo);
+    				pContextMenu->insertItem(sText);
+				}
+			}
+			snd_ctl_close(handle);
+			++iCards;
+		}
+	}
+	
+	// There's always the default device...
+	if (iCards > 0)
+		pContextMenu->insertSeparator();
+	pContextMenu->insertItem(m_pSetup->sDefPresetName);
+
+	// Show the device menu and read selection...
+    int iItemID = pContextMenu->exec(pPushButton->mapToGlobal(QPoint(0,0)));
+    if (iItemID != -1) {
+        sText = pContextMenu->text(iItemID);
+        int iTabPos = sText.find('\t');
+        if (iTabPos >= 0)
+            pLineEdit->setText(sText.left(iTabPos));
+		else
+            pLineEdit->setText(sText);
+    //  settingsChanged();
+    }
+
+    delete pContextMenu;
+}
+
+
+// Interface device selection menu.
+void qjackctlSetupForm::selectInterface (void)
+{
+    deviceMenu(InterfaceComboBox->lineEdit(), InterfacePushButton,
+		AudioComboBox->currentItem());
+}
+
+
+// Input device selection menu.
+void qjackctlSetupForm::selectInDevice (void)
+{
+    deviceMenu(InDeviceComboBox->lineEdit(), InDevicePushButton,
+		QJACKCTL_CAPTURE);
+}
+
+
+// Output device selection menu.
+void qjackctlSetupForm::selectOutDevice (void)
+{
+    deviceMenu(OutDeviceComboBox->lineEdit(), OutDevicePushButton,
+		QJACKCTL_PLAYBACK);
+}
+
+
 // Meta-symbol menu executive.
-void qjackctlSetupForm::symbolMenu( QLineEdit *pLineEdit, QPushButton *pPushButton )
+void qjackctlSetupForm::symbolMenu( QLineEdit *pLineEdit,
+	QPushButton *pPushButton )
 {
     const QString s = "  ";
 
@@ -791,6 +914,9 @@ void qjackctlSetupForm::accept (void)
         m_pSetup->bServerConfig            = ServerConfigCheckBox->isChecked();
         m_pSetup->sServerConfigName        = ServerConfigNameComboBox->currentText();
         m_pSetup->bServerConfigTemp        = ServerConfigTempCheckBox->isChecked();
+        m_pSetup->bQueryShutdown           = QueryShutdownCheckBox->isChecked();
+        m_pSetup->bAliasesEnabled          = AliasesEnabledCheckBox->isChecked();
+        m_pSetup->bAliasesEditing          = AliasesEditingCheckBox->isChecked();
     }
 
     // Save combobox history...
